/********************************************************************
 * ADO DB interface library                                         *
 ********************************************************************
 * This library uses Microsoft ADO v2.7 comes with Microsoft OSes,  *
 * MDAC2.7 installation package.                                    *
 * Be sure about version of MDAC, it must be the 2.7+ otherwise     *
 * MetaTrader terminal can be crashed (by fatal error of DLL).      *
 * Note: Check expert advisor "Common" parameters to be sure that   *
 *       DLL imports are allowed.                                   *
 ********************************************************************/

bool SQLTrace = false;
bool ReconnectNeeded = false;
//import section for mapping some functions
#import "..\\libraries\\MQLAdoDB.dll"

// returns version of MQLAdoDB.dll library
string cAdoDBVersion();

// number of last error of connection
int    cGetAdoDBErrorNumber(int pConnection);

// number of last error of cursor
int    cGetCursorErrorNumber(int pCursorID);

// description of last error for connection
string cGetAdoDBErrorDescription(int pConnection);

// description of last error for cursor
string cGetCursorErrorDescription(int pCursorID);

// establish connection to ADO datasource using connection string
// returns the connection identifier
int    cAdoDBConnect(string pConnectionString);

// close database connection by its identifier
void   cAdoDBDisconnect(int pConnection);

// send query (NON-SELECT statement) to database for execution
// return true - when execution succeeded, false - when failed
bool   cAdoDBExecute(int pConnection, string pQuery);

// creates an cursor based on SELECT statement
// return valuse - cursor identifier
int    cAdoDBCursorOpen(int pConnection, string pQuery);

// closes opened cursor       
void   cAdoDBCursorClose(int pCursorID);

// return number of rows was selected by cursor
int    cAdoDBCursorRows(int pCursorID);

// fetch next row from cursor into current row buffer
// return true - if succeeded, otherwise - false
bool   cAdoDBCursorFetchRow(int pCursorID);

// retrieves the value from current row was fetched by cursor
// by field's number
string cAdoDBGetRowField(int pCursorID, int pField);

// returns the first actual connection idfentifier
int     cAdoDBFirstConnection();

// Reads and returns the key value from standard INI-file
// [SECTION]
// Key = Value
// pFileName - name of file
// pSection  - name of section '[SECTION]'
// pKey      - name of key 'Key ='
// @return   - string value of key 'Value'
string ReadIni (string pFileName, string pSection, string pKey);
string ReadHKCUString (string pPath, string pKey);

#import


//interface variables
int    AdoDBErrorNumber;       // recent MySQL error number
string AdoDBErrorDescription;  // error description

/*
int    CursorErrorNumber;       // recent MySQL error number
string CursorErrorDescription;  // error description
*/
// Interface function AdoDBConnect - make connection to datasource using parameter:
// pConnectionString - DNS name or IP-address
// ------------------------------------------------------------------------------------
// RETURN      - database connection identifier
//               if return value = -1, check MySqlErrorNumber and MySqlErrorDescription
int AdoDBConnect(string pConnectionString)
{
 ClearErrors();
 int result;
 result = cAdoDBConnect(pConnectionString);
 if (SQLTrace) Print ("Connecting to ",pConnectionString," DBID#",result);
 if (result == -1)
    {
     AdoDBErrorNumber = cGetAdoDBErrorNumber(-1);
     AdoDBErrorDescription = cGetAdoDBErrorDescription(-1);
     if ((AdoDBErrorNumber == 7) || (AdoDBErrorNumber == 26)) ReconnectNeeded = true;
     if (SQLTrace) Print ("Connection error #",AdoDBErrorNumber," ",AdoDBErrorDescription);
    }
 else
    {
     ReconnectNeeded = false;
    }
 return (result);
}

// Interface function AdoDBDisconnect - closes connection "pConnection" to database
// When no connection was established - nothing happends
void AdoDBDisconnect(int pConnection)
{
 ClearErrors();
 if (pConnection != -1) cAdoDBDisconnect(pConnection);
 if (SQLTrace) Print ("DBID#",pConnection," disconnected");
}

// Interface function AdoDBExecute - executes SQL query via specified connection
// pConnection - opened database connection
// pQuery      - SQL query
// ------------------------------------------------------
// RETURN      - true : when execution succeded
//             - false: when any error was raised (see AdoDBErrorNumber, AdoDBErrorDescription, AdoDBErrorQuery)
bool AdoDBExecute(int pConnection, string pQuery)
{
 ClearErrors();
 if (SQLTrace) {Print ("DBID#",pConnection,", CMD:",pQuery);}
 if (pConnection == -1) 
    {
     // no connection
     AdoDBErrorNumber = -2;
     AdoDBErrorDescription = "No connection to the database.";
     return (false);
    }
 
 if (!cAdoDBExecute(pConnection, pQuery))
    {
     AdoDBErrorNumber = cGetAdoDBErrorNumber(pConnection);
     AdoDBErrorDescription = cGetAdoDBErrorDescription(pConnection);
     if (SQLTrace) Print ("CMD>",AdoDBErrorNumber, ": ", AdoDBErrorDescription);
     
     if ((AdoDBErrorNumber == 7) || (AdoDBErrorNumber == 26)) ReconnectNeeded = true;

     return (false);
    }
 return (true);
}

// just to clear error buffer before any function started its functionality
void ClearErrors()
{
 AdoDBErrorNumber = 0;
 AdoDBErrorDescription = "No errors.";
}

// return the version of MQLAdoDB.DLL
string AdoDBVersion()
{
 return(cAdoDBVersion());
}

// retrieves the value from current row was fetched by cursor
string AdoDBGetRowField(int pCursorID, int pField)
{
 string result;
 result = cAdoDBGetRowField(pCursorID, pField);
 AdoDBErrorNumber = cGetCursorErrorNumber(pCursorID);
 AdoDBErrorDescription = cGetCursorErrorDescription(pCursorID);
 return (result);
}

// creates an cursor based on SELECT statement
// return value - cursor identifier
int AdoDBCursorOpen(int pConnection, string pQuery)
{
 int result;
 if (SQLTrace) {Print ("DBID#",pConnection,", QRY:",pQuery);}
 ClearErrors();
 result = cAdoDBCursorOpen(pConnection, pQuery);

 if (result == -1)
    {
     AdoDBErrorNumber = cGetAdoDBErrorNumber(pConnection);
     AdoDBErrorDescription = cGetAdoDBErrorDescription(pConnection);
     if (SQLTrace) Print ("QRY>",AdoDBErrorNumber, ": ", AdoDBErrorDescription);
     if ((AdoDBErrorNumber == 7) || (AdoDBErrorNumber == 26)) ReconnectNeeded = true;
    }

 return (result);
}

// closes opened cursor
void AdoDBCursorClose(int pCursorID)
{
 ClearErrors();

 cAdoDBCursorClose(pCursorID);

 AdoDBErrorNumber = cGetCursorErrorNumber(pCursorID);
 AdoDBErrorDescription = cGetCursorErrorDescription(pCursorID);
 if (SQLTrace) 
    {
     if (AdoDBErrorNumber!=0)
        {
         Print ("Cursor #",pCursorID," closing error:", AdoDBErrorNumber, ": ", AdoDBErrorDescription);
        }
     else 
        {
         Print ("Cursor #",pCursorID," closed");
        }
    }

}

// return number of rows was selected by cursor
int AdoDBCursorRows(int pCursorID)
{
 int result;
 result = cAdoDBCursorRows(pCursorID);
 AdoDBErrorNumber = cGetCursorErrorNumber(pCursorID);
 AdoDBErrorDescription = cGetCursorErrorDescription(pCursorID);
 if (SQLTrace) Print ("Cursor #",pCursorID,", rows: ",result);

 return (result);
}

// fetch next row from cursor into current row buffer
// return true - if succeeded, otherwise - false
bool AdoDBCursorFetchRow(int pCursorID)
{
 bool result;
 result = cAdoDBCursorFetchRow(pCursorID);
 AdoDBErrorNumber = cGetCursorErrorNumber(pCursorID);
 AdoDBErrorDescription = cGetCursorErrorDescription(pCursorID);
 return (result); 
}

int AdoDBGetFieldAsInt(int pCursorID, int pField)
{
 return (StrToInteger(AdoDBGetRowField(pCursorID, pField)));
}

double AdoDBGetFieldAsDouble(int pCursorID, int pField)
{
 string val = AdoDBGetRowField(pCursorID, pField);
 StringReplace(val,",",".");
 return (StrToDouble(val));
}

datetime AdoDBGetFieldAsDatetime(int pCursorID, int pField)
{
 return (StrToTime(AdoDBGetRowField(pCursorID, pField)));
}

string AdoDBGetFieldAsString(int pCursorID, int pField)
{
 return (AdoDBGetRowField(pCursorID, pField));
}

int AdoDBFirstConnection()
{
 return (cAdoDBFirstConnection());
}

int MSSQLConnect(string Provider, string Server, string User, string Password, string Database)
{
 string cnn;
 cnn = "Provider="+Provider+";Password="+Password+";User ID="+User+";Initial Catalog="+Database+";Data Source="+Server;
 return (AdoDBConnect(cnn));
}

int MSAccessConnect(string pDataFile)
{
 string cnn;
 cnn = "Provider=Microsoft.Jet.OLEDB.4.0;Password=\"\";Data Source="+pDataFile+";Persist Security Info=True";
 return (AdoDBConnect(cnn));
}
string TimeAsStr(datetime pDate, int pFormat = TIME_DATE|TIME_MINUTES)
{
 string result = "";
 string spc = "";
 if ((pFormat & TIME_DATE)!=0) 
    {
     result = result + IntegerToString(TimeMonth(pDate))+"/"+IntegerToString(TimeDay(pDate))+"/"+IntegerToString(TimeYear(pDate));
     spc = " ";
    }
 if ((pFormat & TIME_SECONDS)!=0)
    {
     result = result + spc + IntegerToString(TimeHour(pDate))+":"+IntegerToString(TimeMinute(pDate))+":"+IntegerToString(TimeSeconds(pDate));
    }
 else if ((pFormat & TIME_MINUTES)!=0)
    {
     result = result + spc + IntegerToString(TimeHour(pDate))+":"+IntegerToString(TimeMinute(pDate));
    }
 return ("#"+result+"#");
}

int OracleConnect(string Provider, string Server, string User, string Password)
{
 string cnn;
 cnn = "Provider="+Provider+";Password="+Password+";User ID="+User+";Data Source="+Server;
 return (AdoDBConnect(cnn));
}

string OracleTimeFormat (datetime pTime)
{
 return ("TO_DATE(\'"+TimeToStr(pTime,TIME_DATE|TIME_SECONDS)+"\',\'YYYY.MM.DD HH24:MI:SS\')");
}

string PostgressTimeFormat (datetime pTime)
{
 return ("TO_TIMESTAMP(\'"+TimeToStr(pTime,TIME_DATE|TIME_SECONDS)+"\',\'YYYY.MM.DD HH24:MI:SS\')");
}

int PostgressConnect(string Provider, string Server, string User, string Password, string Location)
{
 string cnn; 
 cnn = "Provider="+Provider+";Password="+Password+";User ID="+User+";Data Source="+Server+";Location="+Location;
 return (AdoDBConnect(cnn));
}

int PostgressConnect2(string Provider, string Server, string User, string Password, string Location)
{
 string cnn; 
 cnn = "Provider="+Provider+";Password="+Password+";User ID="+User+";Data Source="+Server+";Initial Catalog="+Location+";Extended Properties=\"SSL=allow;\"";
 return (AdoDBConnect(cnn));
}


int PostgressConnect(string Datasource)
{
 return (AdoDBConnect("Provider=MSDASQL.1;Persist Security Info=False;Data Source="+Datasource));
}